
module manchester_decode #
(
	parameter  FREQ = 20
)
(
    output          man_test,
    input    	 	I_clk,
    input    	 	I_rstn,
    
    output   [3:0]  O_rx_cmd,
    output   [19:0] O_rx_addr,
    output         	O_rx_data_st,
    output   [7:0] 	O_rx_data, 
    output         	O_rx_data_vld,   
    output   [7:0]  O_rx_len, //actual +1
    
    output          O_rx_rply_st,  
    input    [7:0]  I_rx_rply_ack,//[5:4]active
    input    [127:0]I_rx_rply_data,//high byte first {byte0,byte1,…,byte15}
    input    [7:0]  I_rx_rply_len,//actual +1
    input    [7:0]  I_rx_rply_m, //actual +1 
    
    inout    	 	IO_man_p,
    inout    	 	IO_man_n
);

//==========================
localparam PREMAX    = FREQ + FREQ/2; //用于采样周期判断
localparam QUAD3FREQ = FREQ/2 + FREQ/4; //重建时钟，防止误反转
localparam HALFTXPRE = FREQ/2-1;//发送半周期
localparam TXPRE     = FREQ-1;//发送周期,1us

localparam CYCMAX     = FREQ-1; 
localparam T1_4CYCMAX = FREQ/4-1;
localparam T1_2CYCMAX = FREQ/2-1;
localparam T3_4CYCMAX = FREQ/2+FREQ/4-2;

localparam WAITREPLYMAX = 0;//回复等待时间,unit:1us , 可根据实际修改时间长度

localparam TIME_XUS = 300*FREQ; //prevent endless loop

localparam CMDTYPE_WR     = 4'b1000;//8
localparam CMDTYPE_RD     = 4'b1001;//9
localparam CMDTYPE_I2CWRM = 4'b0100;//4
localparam CMDTYPE_I2CRDM = 4'b0101;//5
localparam CMDTYPE_I2CWR  = 4'b0000;//0
localparam CMDTYPE_I2CRD  = 4'b0001;//1

localparam ST_IDLE       = 5'd0 ;
localparam ST_SYNCREQ    = 5'd1 ;
localparam ST_RDY        = 5'd2 ;
localparam ST_SYNC2L     = 5'd3 ;
localparam ST_CMDADDR    = 5'd4 ;
localparam ST_SYNCHECK   = 5'd5 ;
localparam ST_LEN        = 5'd6 ;
localparam ST_WDATA      = 5'd7 ;
localparam ST_STOP2H     = 5'd8 ;
localparam ST_STOP2L     = 5'd9 ;
localparam ST_WAITREPLY  = 5'd10;
localparam ST_SYNCREPLY  = 5'd11;
localparam ST_SYNCEND    = 5'd12;
localparam ST_ACK        = 5'd13;
localparam ST_RDATA      = 5'd14;
localparam ST_RPLYM      = 5'd15;
localparam ST_REPLYSTOP  = 5'd16;

//==========================
reg  [4:0] cs;
reg  [4:0] ns;

wire aux_rx;
reg  aux_rx_d1;
reg  aux_rx_d2;

reg  [7:0] precnt;
reg  [3:0] idle_fall_cnt;
     
reg  [5:0] cyccnt;//support max 100MHz. 20MHz~10
reg  [2:0] check_cnt;

reg        dtmp;
reg  [1:0] dtwo;
reg  [1:0] dtwo_d1;
wire       rxd;

reg  [4:0] cmdaddr_bcnt;
reg  [3:0] len_bcnt; 
reg  [7:0] wdata_bcnt; 
reg  [3:0] rxcmd;
reg  [19:0]rxaddr;
reg  [7:0] rxlen;
wire [7:0] rxreallen;
wire [7:0] rxlen_bit;
reg  [7:0] rxwdata; 
reg        rxwdata_valid;
reg        rx_data_st;

wire man_in;
wire man_out;   
reg  tx_en;

reg  [7:0] tx_cycnt;
reg  [7:0] tx_cycnt_d1;//发送数据以此计数为准
reg  [7:0] waitrly_bcnt;
reg  [5:0] synreply_bcnt;
reg  [2:0] synend_bcnt;
reg  [3:0] ack_bcnt;
reg  [2:0] replystop_bcnt;
reg  [7:0] rdata_bcnt;
wire [7:0] rdata_len_bit;
wire [7:0] rdata_len_bit_inv;

reg  [3:0] rplym_bcnt;

reg  [1:0] txdmap;
reg        tx_man_p;

reg        reply_st;

reg [19:0] timexus_cnt;

//==========================
ELVDS_IOBUF TLVDS_IOBUF_inst
(
	.O   (man_in  ),
	.IO  (IO_man_p),
	.IOB (IO_man_n),
	.I   (man_out ),
	.OEN (~tx_en  )
);

//==========================
assign man_out = tx_man_p;

//aux采样2拍
assign aux_rx = man_in;

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        begin
            aux_rx_d1 <=  1'b0;
            aux_rx_d2 <=  1'b0;
        end
    else
        begin
            aux_rx_d1 <=  aux_rx;
            aux_rx_d2 <=  aux_rx_d1;
        end
end

assign O_rx_cmd      = rxcmd;
assign O_rx_addr     = rxaddr;
assign O_rx_len      = rxlen;//rxreallen;
assign O_rx_data_vld = rxwdata_valid;   
assign O_rx_data     = rxwdata; 

//============================
//bit map, 1:2'b10, 0:2'b01
function [1:0] dmap;
    input d;
    begin
        dmap = (d==1'b1) ? 2'b10 : 2'b01;
    end
endfunction

//==========================
//状态机控制
//(1)状态跳转，时序逻辑
always@(posedge I_clk or negedge I_rstn)
begin
	if(!I_rstn)
		cs  <= ST_IDLE;
	else
        cs  <= ns;
end

//(2)状态转移条件判断，组合逻辑
always@(*)
begin
    ns = ST_IDLE;
    case(cs)
        ST_IDLE:
            begin
                if(idle_fall_cnt>=4'd6)
                    ns = ST_SYNCREQ;
                else
                    ns = ST_IDLE;
            end   
        ST_SYNCREQ:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(check_cnt>=3'd7)
                    ns = ST_RDY;
                else
                    ns = ST_SYNCREQ;
            end
        ST_RDY:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(dtwo_d1==2'b11 && dtwo==2'b11)
                    ns = ST_SYNC2L;
                else
                    ns = ST_RDY;
            end
        ST_SYNC2L:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(dtwo_d1==2'b00 && dtwo==2'b00)
                    ns = ST_CMDADDR;
                else
                    ns = ST_SYNC2L;
            end
        ST_CMDADDR:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(cmdaddr_bcnt==5'd24)
                    ns = ST_SYNCHECK;
                else
                    ns = ST_CMDADDR;
            end
        ST_SYNCHECK:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(cyccnt==(T3_4CYCMAX+1'b1))
                    begin
                        if(dtwo==2'b11)
                            ns = ST_STOP2H;
                        else
                            ns = ST_LEN; 
                    end
                else
                    ns = ST_SYNCHECK;
            end
        ST_LEN:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_WR)//write
                    ns = ST_WDATA;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_I2CWRM)//I2c write MOT
                    ns = ST_WDATA;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_I2CWR)//I2c write
                    ns = ST_WDATA;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_RD)//read
                    ns = ST_STOP2H;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_I2CRDM)//I2c read MOT
                    ns = ST_STOP2H;
                else if(len_bcnt==4'd7 && rxcmd==CMDTYPE_I2CRD)//I2c read
                    ns = ST_STOP2H;
                else
                    ns = ST_LEN;
            end
        ST_WDATA:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(wdata_bcnt==rxlen_bit)
                    ns = ST_STOP2H;
                else
                    ns = ST_WDATA;
            end
        ST_STOP2H:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(dtwo_d1==2'b11 && dtwo==2'b11)
                    ns = ST_STOP2L;
                else
                    ns = ST_STOP2H;
            end
        ST_STOP2L:
            begin
                if(timexus_cnt>=TIME_XUS)//200us time out
                    ns <= ST_IDLE;
                else if(dtwo_d1==2'b00 && dtwo==2'b00)
                    ns = ST_WAITREPLY;
                else
                    ns = ST_STOP2L;
            end
        ST_WAITREPLY:
            begin
                if(waitrly_bcnt>=WAITREPLYMAX && tx_cycnt==TXPRE)
                    ns = ST_SYNCREPLY;
                else
                    ns = ST_WAITREPLY;
            end
        ST_SYNCREPLY:
            begin
                if(synreply_bcnt>=6'd30 && tx_cycnt==TXPRE)
                    ns = ST_SYNCEND;
                else
                    ns = ST_SYNCREPLY;
            end
        ST_SYNCEND:
            begin
                if(synend_bcnt>=3'd3 && tx_cycnt==TXPRE)
                    ns = ST_ACK;
                else
                    ns = ST_SYNCEND;
            end
        ST_ACK:
            begin
                if(ack_bcnt>=4'd7 && tx_cycnt==TXPRE && I_rx_rply_ack[5:4]==2'b10) //AUX_DEFER
                    ns = ST_REPLYSTOP;
                else if(ack_bcnt>=4'd7 && tx_cycnt==TXPRE && I_rx_rply_ack[5:4]==2'b00) //AUX_ACK
                    begin
                        if(rxcmd==CMDTYPE_WR || rxcmd==CMDTYPE_I2CWRM || rxcmd==CMDTYPE_I2CWR)
                            ns = ST_REPLYSTOP;
                        else if(rxcmd==CMDTYPE_RD)
                            ns = ST_RDATA;
                        else if(rxcmd==CMDTYPE_I2CRDM || rxcmd==CMDTYPE_I2CRD)
                            begin
                                if(rxlen==8'h7f)
                                    ns = ST_REPLYSTOP;
                                else
                                    ns = ST_RDATA;
                            end
                    end
                else if(ack_bcnt>=4'd7 && tx_cycnt==TXPRE && I_rx_rply_ack[5:4]==2'b01) //AUX_NACK
                    ns = ST_RPLYM;
                else
                    ns = ST_ACK;
            end
        ST_RDATA:
            begin
                if(rdata_bcnt>=(rdata_len_bit-1'b1) && tx_cycnt==TXPRE)
                    ns = ST_REPLYSTOP;
                else
                    ns = ST_RDATA;
            end
        ST_RPLYM:
            begin
                if(rplym_bcnt>=4'd7 && tx_cycnt==TXPRE)
                    ns = ST_REPLYSTOP;
                else
                    ns = ST_RPLYM;
            end
        ST_REPLYSTOP:
            begin
                if(replystop_bcnt>=3'd4)
                    ns = ST_IDLE;
                else
                    ns = ST_REPLYSTOP;
            end
        default:
            begin
                ns = ST_IDLE;
            end
    endcase
end

//(3)状态输出，时序逻辑
//---------------------------------------------------
//连续采6个下降沿，且周期须接近1M
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        precnt <= 8'd0;
    else if(cs == ST_IDLE)
        begin
            if(aux_rx_d2 & !aux_rx_d1) //falling edge
                precnt <= 8'd0;
            else if(precnt>=PREMAX)
                precnt <= 8'd0;
            else
                precnt <= precnt + 1'b1;
        end
    else
        precnt <= 8'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        idle_fall_cnt <= 4'd0;
    else if(cs == ST_IDLE)
        begin
            if(aux_rx_d2 & !aux_rx_d1) //falling edge
                idle_fall_cnt <= idle_fall_cnt + 1'b1;
            else if(precnt>=PREMAX)
                idle_fall_cnt <= 4'd0;
            else
                idle_fall_cnt <= idle_fall_cnt;
        end
    else
        idle_fall_cnt <= 4'd0;
end

//确认6次周期计数值，重建时钟
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        check_cnt <= 3'd0;
    else if(cs == ST_SYNCREQ)
        begin
            if(aux_rx_d2 & !aux_rx_d1) //falling edge
                check_cnt <= check_cnt + 1'b1;
            else if(check_cnt>=3'd7)
                check_cnt <= check_cnt;
            else
                check_cnt <= check_cnt;
        end
    else
        check_cnt <= 3'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        cyccnt <= 6'd0;
    else if(cs == ST_SYNCREQ)
        begin
            if(aux_rx_d2 & !aux_rx_d1) //falling edge
                cyccnt <= 6'd0;
            else
                cyccnt <= cyccnt + 1'b1;
        end
    else
        begin
            if(cs == ST_SYNC2L && (aux_rx_d2 & !aux_rx_d1)) //falling edge
                cyccnt <= 6'd0;
            else if(cs == ST_STOP2L && (aux_rx_d2 & !aux_rx_d1)) //falling edge
                cyccnt <= 6'd0;
            else if((cyccnt>QUAD3FREQ) &&
                    ((aux_rx_d2 & !aux_rx_d1) || (!aux_rx_d2 & aux_rx_d1))) //falling or rising edge
                cyccnt <= 6'd0;
            else if(cyccnt>=CYCMAX)
                cyccnt <= 6'd0;
            else
                cyccnt <= cyccnt + 1'b1;
        end
end


//采集双数
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        dtmp <= 1'b0;
    else if(cyccnt==T1_4CYCMAX)
        dtmp <= aux_rx_d2;
    else
        dtmp <= dtmp;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        dtwo <= 2'b00;
    else if(cyccnt==T3_4CYCMAX)
        dtwo <= {dtmp,aux_rx_d2};
    else
        dtwo <= dtwo;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        begin
            dtwo_d1 <= 2'b00;
        end
    else if(cyccnt==T3_4CYCMAX)
        begin
            dtwo_d1 <= dtwo;
        end
    else
        begin
            dtwo_d1 <= dtwo_d1;
        end
end

assign rxd = (dtwo==2'b10) ? 1'b1 :
             (dtwo==2'b01) ? 1'b0 : 1'b0;


//获取cmd和address
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        cmdaddr_bcnt <= 5'd0;
    else if(cs == ST_CMDADDR)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                cmdaddr_bcnt <= cmdaddr_bcnt + 1'b1;
            else
                cmdaddr_bcnt <= cmdaddr_bcnt;
        end
    else
        cmdaddr_bcnt <= 5'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rxcmd <= 4'd0;
    else if(cs == ST_CMDADDR)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                case(cmdaddr_bcnt)
                    5'd0   : rxcmd[3] <= rxd;
                    5'd1   : rxcmd[2] <= rxd;
                    5'd2   : rxcmd[1] <= rxd;
                    5'd3   : rxcmd[0] <= rxd;
                    default: rxcmd <= rxcmd;
                endcase
            else
                rxcmd <= rxcmd;
        end
    else
        rxcmd <= rxcmd;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rxaddr <= 20'd0;
    else if(cs == ST_CMDADDR)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                case(cmdaddr_bcnt)
                    5'd4   : rxaddr[19] <= rxd;
                    5'd5   : rxaddr[18] <= rxd;
                    5'd6   : rxaddr[17] <= rxd;
                    5'd7   : rxaddr[16] <= rxd;
                    5'd8   : rxaddr[15] <= rxd;
                    5'd9   : rxaddr[14] <= rxd;
                    5'd10  : rxaddr[13] <= rxd;
                    5'd11  : rxaddr[12] <= rxd;
                    5'd12  : rxaddr[11] <= rxd;
                    5'd13  : rxaddr[10] <= rxd;
                    5'd14  : rxaddr[ 9] <= rxd;
                    5'd15  : rxaddr[ 8] <= rxd;
                    5'd16  : rxaddr[ 7] <= rxd;
                    5'd17  : rxaddr[ 6] <= rxd;
                    5'd18  : rxaddr[ 5] <= rxd;
                    5'd19  : rxaddr[ 4] <= rxd;
                    5'd20  : rxaddr[ 3] <= rxd;
                    5'd21  : rxaddr[ 2] <= rxd;
                    5'd22  : rxaddr[ 1] <= rxd;
                    5'd23  : rxaddr[ 0] <= rxd;
                    default: rxaddr <= rxaddr;
                endcase
            else
                rxaddr <= rxaddr;
        end
    else
        rxaddr <= rxaddr;
end

//获取Length
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        len_bcnt <= 4'd0;
    else if(cs == ST_LEN)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                len_bcnt <= len_bcnt + 1'b1;
            else
                len_bcnt <= len_bcnt;
        end
    else
        len_bcnt <= 4'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rxlen <= 8'd0;
    else if(cs == ST_SYNCREQ)
        rxlen <= 8'h7f;
    else if(cs == ST_LEN)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                case(len_bcnt)
                    5'd0   : rxlen[6] <= rxd;
                    5'd1   : rxlen[5] <= rxd;
                    5'd2   : rxlen[4] <= rxd;
                    5'd3   : rxlen[3] <= rxd;
                    5'd4   : rxlen[2] <= rxd;
                    5'd5   : rxlen[1] <= rxd;
                    5'd6   : rxlen[0] <= rxd;
                    default: rxlen <= rxlen;
                endcase
            else
                rxlen <= rxlen;
        end
    else
        rxlen <= rxlen;
end

assign rxreallen = rxlen + 1'b1;
assign rxlen_bit = rxreallen<<3;

//------------------------------
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rx_data_st <= 1'b0;
    else if(cs == ST_LEN && len_bcnt==4'd7 && 
            (rxcmd==CMDTYPE_WR || rxcmd==CMDTYPE_I2CWRM || rxcmd==CMDTYPE_I2CWR))
        rx_data_st <= 1'b1;
    else
        rx_data_st <= 1'b0;
end

assign O_rx_data_st = rx_data_st;

//获取写data
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        wdata_bcnt <= 8'd0;
    else if(cs == ST_WDATA)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                wdata_bcnt <= wdata_bcnt + 1'b1;
            else
                wdata_bcnt <= wdata_bcnt;
        end
    else
        wdata_bcnt <= 8'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rxwdata <= 8'd0;
    else if(cs == ST_WDATA)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1))
                case(wdata_bcnt[2:0])
                    3'd0   : rxwdata[7] <= rxd;
                    3'd1   : rxwdata[6] <= rxd;
                    3'd2   : rxwdata[5] <= rxd;
                    3'd3   : rxwdata[4] <= rxd;
                    3'd4   : rxwdata[3] <= rxd;
                    3'd5   : rxwdata[2] <= rxd;
                    3'd6   : rxwdata[1] <= rxd;
                    3'd7   : rxwdata[0] <= rxd;
                    default: rxwdata <= rxwdata;
                endcase
            else
                rxwdata <= rxwdata;
        end
    else
        rxwdata <= rxwdata;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rxwdata_valid <= 1'd0;
    else if(cs == ST_WDATA)
        begin
            if(cyccnt==(T3_4CYCMAX+1'b1) && wdata_bcnt[2:0]==3'd7)
                rxwdata_valid <= 1'd1;
            else
                rxwdata_valid <= 1'd0;
        end
    else
        rxwdata_valid <= 1'd0;
end

//------------------------------
always@(posedge I_clk or negedge I_rstn)
begin
    if(~I_rstn)
        timexus_cnt <= 20'd0;
    else if((cs == ST_IDLE && idle_fall_cnt>=4'd6) ||
            (cs == ST_SYNCREQ && check_cnt>=3'd7) || 
            (cs == ST_RDY && dtwo_d1==2'b11 && dtwo==2'b11) || 
            (cs == ST_SYNC2L && dtwo_d1==2'b00 && dtwo==2'b00) || 
            (cs == ST_CMDADDR && cmdaddr_bcnt==5'd24) ||
            (cs == ST_SYNCHECK && cyccnt==(T3_4CYCMAX+1'b1)) ||
            (cs == ST_LEN && len_bcnt==4'd8) || 
            (cs == ST_WDATA && wdata_bcnt==rxlen_bit) || 
            (cs == ST_STOP2H && dtwo_d1==2'b11 && dtwo==2'b11))
        timexus_cnt <= 20'd0;
    else   
        timexus_cnt <= timexus_cnt + 1'b1;
end

//------------------
//双向aux信号输出使能
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        tx_en <= 1'b0;
    else if(cs == ST_IDLE)
        tx_en <= 1'b0;
    else if(cs == ST_WAITREPLY)
        tx_en <= 1'b1;
	else  
		tx_en <= tx_en; 
end

//------------------
//进入回复状态，发送周期计数
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        tx_cycnt <= 8'd0;
    else if(tx_en)
        begin
            if(tx_cycnt>=TXPRE)
                tx_cycnt <= 8'd0;
            else
                tx_cycnt <= tx_cycnt + 1'b1;
        end
    else
        tx_cycnt <= 8'd0;
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        tx_cycnt_d1 <= 8'd0;
    else
        tx_cycnt_d1 <= tx_cycnt;
end

//进入回复等待状态
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        waitrly_bcnt <= 8'd0;
    else if(cs == ST_WAITREPLY)
        begin
            if(tx_cycnt>=TXPRE)
                waitrly_bcnt <= waitrly_bcnt + 1'b1;
            else
                waitrly_bcnt <= waitrly_bcnt;
        end
    else
        waitrly_bcnt <= 8'd0;
end

//进入回复Sync reply状态,发送10~16个0+16个0
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        synreply_bcnt <= 6'd0;
    else if(cs == ST_SYNCREPLY)
        begin
            if(tx_cycnt>=TXPRE)
                synreply_bcnt <= synreply_bcnt + 1'b1;
            else
                synreply_bcnt <= synreply_bcnt;
        end
    else
        synreply_bcnt <= 6'd0;
end

//进入回复sync end状态
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        synend_bcnt <= 3'd0;
    else if(cs == ST_SYNCEND)
        begin
            if(tx_cycnt>=TXPRE)
                synend_bcnt <= synend_bcnt + 1'b1;
            else
                synend_bcnt <= synend_bcnt;
        end
    else
        synend_bcnt <= 3'd0;
end

//进入ACK状态
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        ack_bcnt <= 4'd0;
    else if(cs == ST_ACK)
        begin
            if(tx_cycnt>=TXPRE)
                ack_bcnt <= ack_bcnt + 1'b1;
            else
                ack_bcnt <= ack_bcnt;
        end
    else
        ack_bcnt <= 4'd0;
end

//进入回复sync stop状态，最后一个状态
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        replystop_bcnt <= 3'd0;
    else if(cs == ST_REPLYSTOP)
        begin
            if(tx_cycnt>=TXPRE)
                replystop_bcnt <= replystop_bcnt + 1'b1;
            else
                replystop_bcnt <= replystop_bcnt;
        end
    else
        replystop_bcnt <= 3'd0;
end

//进入回复Read Data状态
assign rdata_len_bit = (I_rx_rply_len+1'b1)<<3;
assign rdata_len_bit_inv = 8'd128-1'b1-rdata_bcnt;

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rdata_bcnt <= 8'd0;
    else if(cs == ST_RDATA)
        begin
            if(tx_cycnt>=TXPRE)
                begin
                    if(rdata_bcnt>=(rdata_len_bit-1'b1))
                        rdata_bcnt <= rdata_bcnt;
                    else
                        rdata_bcnt <= rdata_bcnt + 1'b1;
                end
            else
                rdata_bcnt <= rdata_bcnt;
        end
    else
        rdata_bcnt <= 8'd0;
end

//进入回复Write NACK M状态
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        rplym_bcnt <= 4'd0;
    else if(cs == ST_RPLYM)
        begin
            if(tx_cycnt>=TXPRE)
                rplym_bcnt <= rplym_bcnt + 1'b1;
            else
                rplym_bcnt <= rplym_bcnt;
        end
    else
        rplym_bcnt <= 4'd0;
end

//------------------
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        txdmap <= 2'b00;
    else
        begin
            case(cs)
                ST_WAITREPLY: txdmap <= 2'b00;
                ST_SYNCREPLY: txdmap <= dmap(1'b0);
                ST_SYNCEND:
                    begin
                        if(synend_bcnt==3'd0 && tx_cycnt==8'd0)
                            txdmap <= 2'b11;
                        else if(synend_bcnt==3'd1 && tx_cycnt==8'd0)
                            txdmap <= 2'b11;
                        else if(synend_bcnt==3'd2 && tx_cycnt==8'd0)
                            txdmap <= 2'b00;
                        else if(synend_bcnt==3'd3 && tx_cycnt==8'd0)
                            txdmap <= 2'b00;
                        else 
                            txdmap <= txdmap;
                    end
                ST_ACK:
                    begin
                        if(ack_bcnt==4'd0 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[7]);
                        else if(ack_bcnt==4'd1 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[6]);
                        else if(ack_bcnt==4'd2 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[5]);
                        else if(ack_bcnt==4'd3 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[4]);
                        else if(ack_bcnt==4'd4 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[3]);
                        else if(ack_bcnt==4'd5 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[2]);
                        else if(ack_bcnt==4'd6 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[1]);
                        else if(ack_bcnt==4'd7 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_ack[0]);
                        else 
                            txdmap <= txdmap;
                    end
                ST_RDATA:
                    begin
                        if(tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_data[rdata_len_bit_inv]);
                        else 
                            txdmap <= txdmap;
                    end
                ST_RPLYM:
                    begin
                        if(rplym_bcnt==4'd0 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[7]);
                        else if(rplym_bcnt==4'd1 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[6]);
                        else if(rplym_bcnt==4'd2 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[5]);
                        else if(rplym_bcnt==4'd3 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[4]);
                        else if(rplym_bcnt==4'd4 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[3]);
                        else if(rplym_bcnt==4'd5 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[2]);
                        else if(rplym_bcnt==4'd6 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[1]);
                        else if(rplym_bcnt==4'd7 && tx_cycnt==8'd0)
                            txdmap <= dmap(I_rx_rply_m[0]);
                        else 
                            txdmap <= txdmap;
                    end
                ST_REPLYSTOP:
                    begin
                        if(replystop_bcnt==3'd0 && tx_cycnt==8'd0)
                            txdmap <= 2'b11;
                        else if(replystop_bcnt==3'd1 && tx_cycnt==8'd0)
                            txdmap <= 2'b11;
                        else if(replystop_bcnt==3'd2 && tx_cycnt==8'd0)
                            txdmap <= 2'b00;
                        else if(replystop_bcnt==3'd3 && tx_cycnt==8'd0)
                            txdmap <= 2'b00;
                        else 
                            txdmap <= txdmap;
                    end
                default: txdmap <= txdmap;
            endcase
        end
end

always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        tx_man_p <= 1'b0;
    else if(tx_cycnt_d1<=HALFTXPRE)
        tx_man_p <= txdmap[1];
    else if(tx_cycnt_d1>HALFTXPRE)
        tx_man_p <= txdmap[0];
    else
        tx_man_p <= tx_man_p;
end

//------------------------------
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        reply_st <= 1'b0;
    else if(cs == ST_STOP2L && dtwo_d1==2'b00 && dtwo==2'b00)
        reply_st <= 1'b1;
    else
        reply_st <= 1'b0;
end

assign O_rx_rply_st = reply_st;

//-----------------
//for test
assign man_test = tx_en ? man_out : man_in;

endmodule
