module manchester_encode #
(
    parameter  FREQ = 20
)
(
    output            man_test,
    input             I_clk,
    input             I_rstn,
    
    input             I_tx_st, 
    input             I_tx_i2c_nolen,
    input     [3:0]   I_tx_cmd, 
    input     [19:0]  I_tx_addr,
    input     [7:0]   I_tx_len, //actual +1
    input     [127:0] I_tx_data,//high byte first {byte0,byte1,…,byte15} 
   
    output reg        O_tx_rply_data_vld,   
    output    [127:0] O_tx_rply_data,//high byte first {byte0,byte1,…,byte15}
    output reg[7:0]   O_tx_rply_len,
    output reg        O_tx_rply_ack_vld,      
    output    [7:0]   O_tx_rply_ack,//[5:4]active 
    output reg        O_tx_rply_m_vld,      
    output    [7:0]   O_tx_rply_m, //actual +1   
    
    inout             IO_man_p,
    inout             IO_man_n
    
);

localparam P_HALF  = FREQ/2;
localparam P_TWO   = FREQ*2;
localparam P_OAOH  = FREQ/2*3;
localparam P_TAOH  = FREQ/2*5;
localparam P_1QUAR = FREQ/4;
localparam P_3QUAR = P_1QUAR *3;
  
localparam TIME_XUS = 300*FREQ; //prevent endless loop

localparam CMDTYPE_WR     = 4'b1000;//8
localparam CMDTYPE_RD     = 4'b1001;//9
localparam CMDTYPE_I2CWRM = 4'b0100;//4
localparam CMDTYPE_I2CRDM = 4'b0101;//5
localparam CMDTYPE_I2CWR  = 4'b0000;//0
localparam CMDTYPE_I2CRD  = 4'b0001;//1

localparam[3:0]IDLE         = 4'd0  ;
localparam[3:0]SYNC         = 4'd1  ;
localparam[3:0]SYNC_H       = 4'd2  ;
localparam[3:0]SYNC_L       = 4'd3  ;
localparam[3:0]CMD_ADDR     = 4'd4  ;
localparam[3:0]LEN          = 4'd5  ;
localparam[3:0]WR           = 4'd6  ;
localparam[3:0]WAIT         = 4'd7  ;
localparam[3:0]JUDGE        = 4'd8  ;
localparam[3:0]WAIT_H2L2    = 4'd9  ;
localparam[3:0]ACK          = 4'd10 ;
localparam[3:0]RD           = 4'd11 ;
localparam[3:0]RPLY_WRM     = 4'd12 ;

reg  I_tx_en_dly;
wire st_real;

reg [4:0]cnt_judege;
reg [6:0]cnt_edge;//沿之间计数
reg [3:0]state; 
reg [7:0]hcnt;  
reg [9:0]vcnt;
wire sync_done ;
wire sync_hend ;
wire sync_lend ;
wire addr_end  ;
wire len_end   ;
wire wr_end    ;
wire l2_end    ;
wire ack_end   ;
wire rd_end    ;
wire rply_wrm_end  ;
reg flg_sync    ;
reg flg    ;
reg [127:0]d_shift;
wire edge0;
wire time_out    ;
wire sync_err    ;

reg [19:0] timexus_cnt;

wire[8:0]data_index   ;

wire     rec;
reg [3:0]rec_dly;

reg[15:0] ack_rec;
reg[255:0]data_rec;
reg[15:0] wrm_rec;
wire      aux_in_r;
reg[8:0] vcnt_d_rec;
 
wire low_l;
wire high_l;

wire syn_high;

wire man_in;
reg  man_out;
reg  tx_en;

//==============================================================
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)
        I_tx_en_dly <= 1'b0;
    else  
        I_tx_en_dly <= I_tx_st;
end 
assign st_real = I_tx_st & ~I_tx_en_dly;

assign data_index = (state == RPLY_WRM) ? 9'd16 : ((I_tx_len+1'b1)<<4);

assign sync_done  =  vcnt == 10'd63  && hcnt ==  P_HALF-1'b1 && state == SYNC;
assign sync_hend  =  vcnt == 10'd3   && hcnt ==  P_HALF-1'b1 && state == SYNC_H;
assign sync_lend  =  vcnt == 10'd3   && hcnt ==  P_HALF-1'b1 && state == SYNC_L; 
assign addr_end   =  vcnt == 10'd47  && hcnt ==  P_HALF-1'b1 && state == CMD_ADDR;
assign len_end    =  vcnt == 10'd15  && hcnt ==  P_HALF-1'b1 && state == LEN;
assign wr_end     =  vcnt == data_index-1'b1 && hcnt ==  P_HALF-1'b1 & state == WR;
 
always@(posedge I_clk or negedge I_rstn) 
begin
    if(!I_rstn)begin
        state         <= IDLE; 
    end
    else case(state)
        IDLE:
            if(st_real)  
                state    <= SYNC;  
        SYNC:
            if(sync_done)
                state   <= SYNC_H;
        SYNC_H:
            if(sync_hend)
                state   <= SYNC_L;
        SYNC_L:
            if(sync_lend)
                if(flg_sync)    
                    state   <= WAIT;
                else
                    state   <= CMD_ADDR;
        CMD_ADDR:
            if(addr_end)
                if(I_tx_i2c_nolen)
                    state   <= SYNC_H;
                else
                    state   <= LEN;
        LEN:
            if(len_end)
                if(I_tx_cmd==CMDTYPE_RD || I_tx_cmd==CMDTYPE_I2CRDM || I_tx_cmd==CMDTYPE_I2CRD)
                    state <= SYNC_H;
                else
                    state <= WR;
        WR:
            if(wr_end)
                state <= SYNC_H;
                
        WAIT:
            if(time_out)
                state <= IDLE;
            else if(edge0)
                state <= JUDGE;
        JUDGE:
            if(timexus_cnt>=TIME_XUS)//200us time out
                state <= IDLE;
            else if(sync_err)
                state <= JUDGE;
            else if(cnt_judege>=20)
                state <= WAIT_H2L2;
            else
                state <= JUDGE;
        WAIT_H2L2:
            if(timexus_cnt>=TIME_XUS)//200us time out
                state <= IDLE;
            else if(l2_end) 
                state <= ACK;
        ACK :
            if(timexus_cnt>=TIME_XUS)//200us time out
                state <= IDLE;
            else if(ack_end)
                if(I_tx_cmd==CMDTYPE_RD || 
                   (I_tx_cmd==CMDTYPE_I2CRDM && ~I_tx_i2c_nolen) ||
                   (I_tx_cmd==CMDTYPE_I2CRD && ~I_tx_i2c_nolen))
                    state <=RD;
                else if(O_tx_rply_ack[5:4]==2'b01)
                    state <=RPLY_WRM;
                else
                    state <= IDLE;
        RD:
            if(timexus_cnt>=TIME_XUS)//200us time out
                state <= IDLE;
            else if(rd_end)
                state <= IDLE;
            else if(syn_high)
                state <= IDLE;
        RPLY_WRM:
            if(timexus_cnt>=TIME_XUS)//200us time out
                state <= IDLE;
            else if(rply_wrm_end)
                state <= IDLE;
        default:
            state <= IDLE;
    endcase
        
end 

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        hcnt <= 8'd0;
    else if(st_real|hcnt==P_HALF-1'b1)    
        hcnt <= 8'd0;
    else  
        hcnt <= hcnt + 1'b1;
end

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        vcnt <= 10'd0;
    else if(sync_done | st_real |sync_hend|sync_lend|addr_end|wr_end|len_end )
        vcnt <= 10'd0;
    else if(hcnt==P_HALF-1'b1)    
        vcnt <= vcnt + 1'b1; 
end

always@(posedge I_clk or negedge I_rstn)
begin
    if(~I_rstn)
        timexus_cnt <= 20'd0;
    else if((state==WAIT) || (state==JUDGE && cnt_judege>=20) | l2_end | ack_end)
        timexus_cnt <= 20'd0;
    else   
        timexus_cnt <= timexus_cnt + 1'b1;
end


always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        flg <= 1'b0;
    else if(sync_lend)    
        flg <= 1'd0;
    else if(hcnt==P_HALF-1'b1)
        flg <= ~flg;
end 


always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        flg_sync <= 1'b0;
    else if(st_real)    
        flg_sync <= 1'd0;
    else if(addr_end)
        flg_sync <= 1'd1;
end

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        man_out <= 1'b0;
    else if(state == SYNC)    
        man_out <= flg;
    else if(state == SYNC_H ) 
        man_out <= 1'd1;
    else if(state == SYNC_L ) 
        man_out <= 1'd0;
    else if(state == CMD_ADDR || state == LEN || state == WR ) 
        man_out <= d_shift[127] ? ~flg :flg ; 
end


always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        d_shift <= 1'b0;
    else if(len_end && (I_tx_cmd==CMDTYPE_WR || I_tx_cmd==CMDTYPE_I2CWRM))    
        d_shift <= I_tx_data;
    else if(sync_lend)
        d_shift <= {I_tx_cmd,I_tx_addr,I_tx_len,96'h0};
    else if(flg && hcnt == P_HALF-1'b1) 
        d_shift <= d_shift<<1;
end


//回复相关
assign rec = man_in;

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        rec_dly <= 4'd0;
    else    
        rec_dly <= {rec_dly[2:0],rec};
end

assign rise = rec_dly[2]& ~rec_dly[3];
assign fall = ~rec_dly[1]& rec_dly[2];
assign aux_in_r = rec_dly[2];
assign edge0 =  rise|fall;


assign high_l = aux_in_r   && cnt_edge == P_3QUAR;
assign low_l  = ~aux_in_r  && cnt_edge == P_3QUAR;

assign syn_high = aux_in_r   && cnt_edge == P_OAOH;

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        vcnt_d_rec <= 9'd0;
    else if(l2_end | ack_end)
        vcnt_d_rec <= 9'd0;
    else if(rise || low_l||high_l||fall)    
        vcnt_d_rec <= vcnt_d_rec + 1'b1; 
end

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        data_rec <= 256'd0;
    else if(l2_end || ack_end )    
        data_rec <= 256'd0; 
    else if(rise || low_l)
        data_rec <= {data_rec[254:0],1'd0};
    else if(fall || high_l)
        data_rec <= {data_rec[254:0],1'd1};
end    

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        ack_rec <= 16'd0;
    else if(l2_end)    
        ack_rec <= 16'd0; 
    else if(rise || low_l)
        ack_rec <= {ack_rec[14:0],1'd0};
    else if(fall || high_l)
        ack_rec <= {ack_rec[14:0],1'd1};
end   

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        wrm_rec <= 16'd0;
    else if(ack_end)    
        wrm_rec <= 16'd0; 
    else if(rise || low_l)
        wrm_rec <= {wrm_rec[14:0],1'd0};
    else if(fall || high_l)
        wrm_rec <= {wrm_rec[14:0],1'd1};
end 

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        cnt_edge <= 7'd0;
    else if(edge0)    
        cnt_edge <= 7'd0;
    else  
        cnt_edge <= cnt_edge + 1'b1;
end

always@(posedge I_clk or negedge I_rstn)
begin
    if(~I_rstn)
        cnt_judege <= 5'd0;
    else if(state == JUDGE)
        begin
            if(sync_err)
                cnt_judege <= 5'd0;
            else
                cnt_judege <= edge0 ? cnt_judege + 1'b1 : cnt_judege;
        end
    else 
        cnt_judege <= 5'd0;
end


assign time_out     = vcnt == 10'd600 && hcnt ==  P_HALF-1'b1 && state == WAIT; //300us
assign sync_err     = (cnt_edge > P_3QUAR || cnt_edge < P_1QUAR) && edge0 && state == JUDGE;
// assign l2_end       = rise && cnt_edge > P_TWO  && state == WAIT_H2L2;
assign l2_end       = ~aux_in_r && cnt_edge > P_TWO  && state == WAIT_H2L2;
assign ack_end      = ack_rec[14] && state == ACK;
assign rd_end       = vcnt_d_rec == data_index && state == RD;
assign rply_wrm_end = vcnt_d_rec == data_index && state == RPLY_WRM;

//---------
genvar k;
for(k=0;k<=7;k=k+1)begin
    assign    O_tx_rply_ack[k] = {ack_rec[2*k+1],ack_rec[2*k]} == 2'b01 ? 1'b0 :1'b1;
end

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        O_tx_rply_ack_vld <= 1'd0;
    else     
        O_tx_rply_ack_vld <= ack_end; 
end

//---------
genvar m;
for(m=0;m<=7;m=m+1)begin
    assign    O_tx_rply_m[m] = {wrm_rec[2*m+1],wrm_rec[2*m]} == 2'b01 ? 1'b0 :1'b1;
end

always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        O_tx_rply_m_vld <= 1'd0;
    else     
        O_tx_rply_m_vld <= rply_wrm_end; 
end
 
 //---------
genvar i;
for(i=0;i<=127;i=i+1)begin
    assign    O_tx_rply_data[i] = {data_rec[2*i+1],data_rec[2*i]} == 2'b01 ? 1'b0 :1'b1;
end

always@(posedge I_clk or negedge I_rstn)
begin
    if(~I_rstn)
        O_tx_rply_data_vld <= 1'd0;
    else if(state == RD && syn_high)
        O_tx_rply_data_vld <= 1'd1;
    else     
        O_tx_rply_data_vld <= rd_end; 
end

always@(posedge I_clk or negedge I_rstn)
begin
    if(~I_rstn)
        O_tx_rply_len <= 8'd0;
    else if(ack_end)
        O_tx_rply_len <= 8'd0;
    else if(state == RD && syn_high)
        O_tx_rply_len <= vcnt_d_rec>>4;
    else if(rd_end)
        O_tx_rply_len <= vcnt_d_rec>>4;
    else     
        O_tx_rply_len <= O_tx_rply_len; 
end


//------------OUT
always@(posedge I_clk or negedge I_rstn)begin
    if(~I_rstn)
        tx_en <= 1'd0;
    else if(st_real)    
        tx_en <= 1'd1;
    else if(sync_lend && flg_sync) 
        tx_en <= 1'd0;
end

//==========================
ELVDS_IOBUF TLVDS_IOBUF_inst
(
	.O   (man_in  ),
	.IO  (IO_man_p),
	.IOB (IO_man_n),
	.I   (man_out ),
	.OEN (~tx_en  )
);

//-----------------
//for test
assign man_test = tx_en ? man_out : man_in;

endmodule

